#include<stdio.h>
#include<stdlib.h> 
#include "dict.h"    /* Defines types, routines for dictionary  */ 

/*------------------------------------------------------------*/
/* This is a dictionary driver for the DIMACS tests.  It      */
/* reads command lines and performs procedure calls, and      */
/* records statistics about the run.  The input/output        */
/* assumes that in_type is a string and ky_type is an int or  */
/* a string (depending on compile-time switch).               */ 
/* The switch appears in instantiate.h                        */
/*  ccm 5/96                                                  */
/*------------------------------------------------------------*/

#define MAXNAMES 100000          /* Max user names in input*/ 

#define DETAIL 1               /* Set 0..10 for how much detail to report */ 

it_type itemindex[MAXNAMES+1];   /* Maps user names (integers) to items  */ 
int     inx;                     /* index to itemindex                   */
int     N;                       /* size of dictionary                    */

dict_ptr  D;                       /* The dictionary                   */
ky_type key;                     /* keys                             */
it_type item;                    /* item locator                     */
in_type dummy;                   /* string for info part             */ 

struct cell *node; 

#define CMD_SIZE 10              /* User command interpreter stuff      */
typedef char cmdtype[3];         
cmdtype cmdtable[CMD_SIZE+1];    /* table of user commands               */
cmdtype cmd;                     /* user command                         */
int count;                       /* counts fields in input               */

/*-------------tabinit------------------------------------------*/
/* Initializes the table of user commands, and sets defaults    */
void tabinit()
{
  strcpy(cmdtable[0], "sentinal.spot");
  strcpy(cmdtable[1], "dch");      /* header line*/
  strcpy(cmdtable[2], "com");      /* comment */
  strcpy(cmdtable[3], "ins");      /* insert  */ 
  strcpy(cmdtable[4], "lkp");      /* lookup */
  strcpy(cmdtable[5], "dlk");      /* delete by key      */ 
  strcpy(cmdtable[6], "dli");      /* delete by item ref */ 
  strcpy(cmdtable[7], "kyv");      /* keyval */
  strcpy(cmdtable[8], "inv");      /* infoval */
  strcpy(cmdtable[9], "siz");      /* size */
  strcpy(cmdtable[10],"clr");     /* clear */
}/*tabinit*/

/*-------------lookup-------------------------------------------*/
/* Look up user command in command table, return integer        */
int cmd_lookup (cmdtype cmd) 
{
  int i;
  int stop;
  strncpy(cmdtable[0], cmd,3);  /*sentinel*/ 
  stop = 1;
  for (i = CMD_SIZE ; stop != 0; i--) {
               stop = strncmp(cmdtable[i], cmd, 3);
	     } 
  return(i+1);
}/*lookup*/ 

/*----------printem----------------------------------------------*/
void printem(struct cell *ptr) 
{
   if (ptr == NULL) return;
   else { 
#ifdef INTEGER_KEY
    printf("%d %s\n", ptr->dkey, ptr->dinfo);  
    printf("  %d  lchild: \n", ptr->dkey); printem(ptr->lch);  
    printf("  %d  rchild: \n", ptr->dkey); printem(ptr->rch);
#endif
#ifdef STRING_KEY
    printf("%s %s\n", ptr->dkey, ptr->dinfo);  
    printf("  %s  lchild: \n", ptr->dkey); printem(ptr->lch);  
    printf("  %s  rchild: \n", ptr->dkey); printem(ptr->rch);
#endif 
    
  }
 }
/*---------main-------------------------------------------------*/
/* Driver reads command lines, calls routines, reports          */
main()
{
char buf[100];  /*input line buffer */ 
int index;    

tabinit();     
while (scanf("%s", cmd) != EOF) {
        fgets (buf, sizeof(buf), stdin);     
        index = cmd_lookup (cmd); 
        switch(index) {
        case 0 : { printf("%s: Unknown Command\n", cmd);
                   break;
		 }
        case 1 : { /* dch */
                   sscanf ( buf, "%d \n", &N);
                   D = construct_dict();
                   if (DETAIL == 1) printf("dch %d \n", N); 
                   if (N > MAXNAMES) {
                          printf("N too big, please recompile\n");
                          exit;
			}
                   break;
		 }
        case 2 : { /*com*/ 
                   break;
                 }
        case 3 : { /*ins*/
#ifdef INTEGER_KEY
                   count = sscanf( buf,  "%d  %d\n", &key, &inx); 
#endif
#ifdef STRING_KEY
                   count = sscanf( buf,  "%s  %d\n", key, &inx); 
#endif  
                   if (count == 1) { 
                       strcpy(dummy, "xxx\0"); 
                       inx = 0; 
		     }else 
                   if (count == 2) {
                       sprintf( dummy, "xxx%d", inx); 
                       if (inx >= MAXNAMES) {
                          printf ("item name too big, please recompile\n");
                          exit(1); 
                       } 
		     }/*count */ 

                   item = insert ( D, key, dummy); 
                   if (inx != 0) itemindex[inx] = item;
                   if (DETAIL == 1) {
#ifdef INTEGER_KEY
                        printf("ins %d %d : %d %s\n",
                             key,inx, keyval(D, item), infoval(D, item)); 
#endif 
#ifdef STRING_KEY
                        printf("ins %s %d : %s %s\n",
                             key,inx, keyval(D, item), infoval(D, item)); 
#endif
		      }/*if detail*/ 
                   break;
                 } 
        case 4:  { /*lkp*/ 
                   inx = 0; 
#ifdef INTEGER_KEY
                   count = sscanf ( buf, "%d  %d", &key, &inx); 
#endif
#ifdef STRING_KEY
                   count = sscanf ( buf, "%s  %d", key, &inx); 
#endif 
	           item = lookup(D, key );
                   if ((item != NULL_ITEM) && (count == 2) && (inx != 0))
                        { itemindex[inx] = item; } 

                   if (DETAIL == 1) {
#ifdef INTEGER_KEY
                       if (item == NULL_ITEM) 
                            printf("lkp %d : not found\n", key);
                       else printf("lkp %d %d: found %d %s\n", 
                                    key, inx, keyval(D, item), 
                                              infoval(D, item) ); 
#endif 
#ifdef STRING_KEY
                       if (item == NULL_ITEM) 
                            printf("lkp %s : not found\n", key);
                       else printf("lkp %s %d: found %s %s \n", 
                             key, inx, keyval(D, item), infoval(D, item)); 
#endif 

		     } 
                   break; 
                 }
        case 5:  { /*dlk*/
#ifdef INTEGER_KEY
                   sscanf( buf, "%d", &key); 
#endif
#ifdef STRING_KEY
                   sscanf( buf, "%s", key); 
#endif 
                   delete(D, key );
                   if (DETAIL == 1) 
#ifdef INTEGER_KEY
                   printf("dlk %d \n", key);
#endif
#ifdef STRING_KEY
                   printf("dlk %s \n", key);
#endif

                   break;
                 }
        case 6:  { /* dli*/
                   sscanf ( buf , "%d", &inx);
                   item = itemindex[inx];
                   del_item(D, item); 
                   if (DETAIL == 1) printf("dli %d \n",inx); 
                   break; 
		 }
	case 7:  { /*kyv*/
                   sscanf (buf, "%d", &inx);
                   item = itemindex[inx];

#ifdef INTEGER_KEY
                   key = keyval (D, item);
#endif 
#ifdef STRING_KEY 
                   strcpy (key, *keyval(D, item)); ; 
#endif

                   if (DETAIL == 1 )
#ifdef INTEGER_KEY
                   printf ("kyv %d : %d\n" , inx, key); 
#endif
#ifdef STRING_KEY
                   printf ("kyv %d : %s\n" , inx, key); 
#endif

                   break; 
                 }
        case 8:  { /*inv*/
                   sscanf (buf ,"%d", &inx);
                   item = itemindex[inx];
                   strcpy(dummy, *infoval(D, item)); 
                   if (DETAIL == 1) printf ("inv %d : %3s\n", inx, dummy);
                   break;
                 }
        case 9:  { /* siz */
                    inx = size(D);
                    if (DETAIL == 1) printf ("siz :  %d\n", inx); 
                    break;
		  } 
        case 10:  { /* clr */
                    clear (D);
                    if (DETAIL == 1) printf("clr : \n"); 
                    break; 
                  }
	}/*end switch*/
#ifdef DEBUG 
 printf("Dict looks like: \n"); 
 node = D->mydroot; 
 printem(node); 
#endif
    }/*while scanf*/
    destruct_dict(D); 
}/*main*/ 
