#include<stdio.h>
#include<stdlib.h>

/*  This is a simple dictionary package built on a bst to             */
/*     demonstrate the DIMACS Challenge  interface.                   */
/*  Item references are persistent and correct throughout the         */
/*     lifetime of the  item.                                         */
/*  Put #define STRING_KEY in instantiate.h to modify the keyval      */
/*     function declaration for strings.                              */
/*   C. McGeoch 5/96                                                  */

#include "dc_types.h" 
          /* Include this to get type and macro declarations         */ 

/*------construct_pq-----------------------------------*/
dict_ptr construct_dict() 
{
dict_ptr dhead;
  dhead = (dict_ptr) malloc(sizeof(mydict));
  dhead->mydsize = 0;
  dhead->mydroot = NULL; 
  return(dhead);
 }/*construct_dict*/ 

/*------insert-----------------------------------------*/
it_type insert (dict_ptr D, ky_type key, in_type in) 
{ 
it_type tmp, curr;

  tmp = (celltype *) malloc(sizeof(celltype));
  KY_ASSIGN(tmp->dkey, key);
  IN_ASSIGN(tmp->dinfo,in);

  tmp->lch = NULL;
  tmp->rch = NULL;
  D->mydsize++;

  if(D->mydroot == NULL) { /* first insert*/
       D->mydroot = tmp;
       tmp->parent = NULL; 
       return(tmp);
     }/*first*/
   else {/* tree exists*/ 
     curr = D->mydroot; 
     while ( 1 ) { 
           /* inv: curr points to parent of possible spot */ 
           if (KY_COMPARE (curr->dkey, key) < 0) {/*go right*/
              if (curr->rch != NULL)  
                  curr = curr->rch;
	      else {
                   curr->rch = tmp;
                   tmp->parent = curr; 
                   return(tmp);
		 }
	    }/*right*/ 
	   else if(KY_COMPARE(curr->dkey , key) > 0) {/* go left*/
	     if (curr->lch != NULL) 
	           curr = curr->lch;
	     else {
	           curr->lch = tmp;
                   tmp ->parent = curr; 
                   return(tmp);
             }   
	   }/*left*/
	   else {
                 printf("tried to insert redundant key (ignored)\n");
                 free(tmp); 
                 return(curr); 
	       }/*else*/

	 }/*while*/
   }/*else tree exists*/
} /*insert*/ 

/*-------lookup--------------------------------------*/
it_type lookup (dict_ptr D, ky_type key) 
{
it_type curr;
double comp; 

   if (D->mydroot == NULL) return(NULL_ITEM);
   else { /*tree exists*/ 
       curr = D->mydroot;
       while ( 1) {
          if (curr == NULL) return(NULL_ITEM);
          else {
               comp = KY_COMPARE(curr->dkey, key); 
               if (comp < 0) curr = curr->rch;
               else if (comp > 0) curr = curr->lch;
               else return(curr);
	     }/*else*/
	}/*while*/
     }/*else*/
}/*lookup*/ 

/*-------del_item------------------------------------*/
void del_item(dict_ptr D, it_type item)
{ 
it_type replacer, curr, par; 
it_type ip, il,ir;
it_type rp, rl,rr; 

   ip = item->parent;   
   il = item->lch;
   ir = item->rch;

  /* find leftmost node in right subtree of item*/
  curr = item->rch; 

  if (curr == NULL) { /*no right subtree*/
     /* items parent points to items lchild */
     if (ip == NULL) {D->mydroot = il; }
     else {
        if (ip->rch == item) {ip->rch = il;  }
        else 
        if (ip->lch == item) {ip->lch = il;  } 
      }
     /* lchild points to new parent */
     if (il != NULL) il->parent = item->parent;
   }/* no right subtree */

  else { /* has right subtree, find leftmost as replacer */ 
        while (curr->lch != NULL) curr = curr->lch; 
        replacer = curr;  
        rp = replacer->parent; 
        rl = replacer->lch;   
        rr = replacer->rch;   
         
        /* replacer new parent links*/ 
        replacer->parent = ip; 
        if (ip == NULL) { D->mydroot = replacer;}
        else { if      (ip->lch == item) {ip->lch = replacer;}
               else if (ip->rch == item) {ip->rch = replacer;} 
	     }

        /* replacer new lch links*/ 
        replacer->lch    = il;
        if (il != NULL) il->parent = replacer;

        /* replacer new rch links IF it is not items rch */ 
        if (replacer != ir) {
            replacer->rch = ir;  
            if (ir != NULL) ir->parent = replacer;            
            rp->lch = rr; 
            if (rr != NULL) rr->parent = rp;            
	  } /*otherwise keep old values*/ 

      }/* else has subtree*/

        D->mydsize--; 
        free(item); 
        return; 
}/*del_item*/ 


/*-------delete---------------------------------------*/ 
void delete (dict_ptr D, ky_type key) 
{ 
it_type item;

    item = lookup (D, key);
    del_item( D, item ); 

}/*delete*/ 

/*-------keyval---------------------------------------*/
#ifdef STRING_KEY 
ky_type *keyval (dict_ptr D, it_type item) 
{
  return(&(item->dkey));
 }/*keyval*/
#else 
ky_type keyval (dict_ptr D, it_type item) 
{
  return(item->dkey);
 }/*keyval*/
#endif


/*-------infoval----------------------------------------*/
in_type *infoval (dict_ptr D, it_type item) 
{
   return(&(item->dinfo)); 
}/*infoval*/

/*-------size-------------------------------------------*/
int size (dict_ptr D) 
{
    return (D->mydsize);
  } /*size*/ 
/*-------clear------------------------------------------*/
void removem (it_type curr)
{ 
  if (curr == NULL) return;
  else{
    removem (curr->lch);
    removem (curr->rch);
    free(curr);
  }
}/*removem*/ 

void clear (dict_ptr D) 
{
it_type curr;
   curr = D->mydroot; 
   removem( curr ); 
   return; 

}/*clear*/
/*--------destruct_dict-----------------------------------*/
dict_ptr destruct_dict (dict_ptr D)
{
    clear(D);
    free(D); 
    return(NULL);
} /*destruct_dict*/


